#!/bin/bash

shopt -s expand_aliases

if test -t 1
then

    alias .echo_red='if :; then read; echo -ne "\e[0;31m"; echo -n $REPLY; echo -ne "\e[0;00m"; fi <<<'
    alias .echo_green='if :; then read; echo -ne "\e[0;32m"; echo -n $REPLY; echo -ne "\e[0;00m"; fi <<<'
    alias .echo_yellow='if :; then read; echo -ne "\e[0;33m"; echo -n $REPLY; echo -ne "\e[0;00m"; fi <<<'
    alias .echo_blue='if :; then read; echo -ne "\e[1;36m"; echo -n $REPLY; echo -ne "\e[0;00m"; fi <<<'

else

    alias .echo_red='echo -n'
    alias .echo_green='echo -n'
    alias .echo_yellow='echo -n'
    alias .echo_blue='echo -n'

fi

alias info=": #"

function error()
{
    local __message="$@"

    .echo_red "ERROR"
    echo "   : $__message"
    if declare -p PARSE_OPTIONS_SECTION >& /dev/null
    then
        print_help
    fi
    exit 2
}

function warning()
{
    local __message="$@"

    .echo_yellow "WARNING"
    echo " : $__message"
    return 0
}

function info()
{
    local __message="$@"

    .echo_green "INFO"
    echo "    : $__message"
    return 0
}

function check_tool()
{
    local __tool="$1"
    local __path="${2:-""}"

    if [ $# == 1 ]
    then
        if ! which $__tool >& /dev/null
        then
            error "There is no" \'$__tool\' "in $PATH"
        fi
    else
        if ! test -e $__path/$__tool
        then
            error "There is no" \'$__tool\' "in $__path"
        fi
    fi
    return 0
}
check_tool which

function is_mountpoint()
{
    local __dir=$1
    local __pdir
    local __dir_device
    local __pdir_device

    test ! -L $__dir || return 1
    test -d $__dir || return 1
    test "$__dir" != "/" || return 1

    __pdir=$( dirname $__dir )
    __dir_device=$( stat -c %d $__dir )
    __pdir_device=$( stat -c %d $__pdir )
    test $__dir_device != $__pdir_device || return 1

    return 0
}

# senior = non-nested
function is_senior_mountpoint()
{
    local __dir=$1
    local __dirname

    is_mountpoint $__dir || return 1
    __dirname=$( dirname $__dir )
    while [ "$__dirname" != "/" ]
    do
        if is_mountpoint $__dirname
        then
            return 1
        fi
        __dirname=$( dirname $__dirname )
    done

    return 0
}

function is_include_suitable()
{
    local __file=$1

    if [ "$__file" == "/dev" ] || [ "${__file:0:5}" == "/dev/" ]
    then
        return 1
    fi
    if [ "$__file" == "/proc" ] || [ "${__file:0:6}" == "/proc/" ]
    then
        return 1
    fi
    if [ "$__file" == "/sys" ] || [ "${__file:0:5}" == "/sys/" ]
    then
        return 1
    fi
    return 0
}

declare -r SCRIPT=$( readlink -f $0 )
function print_help()
{
    if declare -p __SHOW_ACTION__ >& /dev/null
    then
        printf "\npack_guestfs.sh packes guest file system, located in /\n\n"
    else
        printf "\n"
    fi
    printf "Usage:\n"
    printf "$SCRIPT [options]\n\n"
    printf "Options:\n"
    printf "\t-h, -H, --help                    print this help\n"
    printf "\t-v, --verbose                     turn on verbose mode\n"
    printf "\t--include <file/dir>              include file or directory in guest file system archive\n"
    printf "\t--exclude <file/dir>              exclude file or directory from guest file system archive\n"
    printf "\n"
    printf "Tricks:\n"
    printf "\tBy default, script excludes mount points from archive, you may use option --include to change script's behaviour.\n"
    printf "\tDirectories /dev, /proc, /sys are always excluded from archive, option --include does not affect them.\n"
    printf "\n"
    return 0
}

#############################

declare -r USER_ID=$( id -u )
test $USER_ID == 0 || error "You must be root-user to pack guest file system"

# Parsing options
declare -a EXCLUDE_LIST=()
declare -i EXCLUDE_LIST_POS=0
declare -a INCLUDE_LIST=()
declare -i INCLUDE_LIST_POS=0
declare PARSE_OPTIONS_SECTION=
while [ $# != 0 ]
do
    case $1 in
        -h | -H | --help )
            __SHOW_ACTION__= print_help
            exit 0
            ;;
        -v | --verbose )
            unalias info >& /dev/null
            shift
            ;;
        --include )
            test $# -gt 1 || error "File or directory to include in guest file system is not specified"
            FILE=$2
            # Deleting trailing slashes
            while [ "${FILE%/}" != "$FILE" ]
            do
                FILE="${FILE%/}"
            done
            shift 2
            if [ -e $FILE ]
            then
                if is_include_suitable $FILE
                then
                    INCLUDE_LIST[$(( INCLUDE_LIST_POS++ ))]=$FILE
                else
                    warning "Include option for file $FILE will be ignored (directories /dev, /proc, /sys are always excluded from archive)"
                fi
            else
                warning "Include option for file $FILE will be ignored (file does not exist)"
            fi
            ;;
        --exclude )
            test $# -gt 1 || error "File or directory to exclude from guest file system is not specified"
            FILE=$2
            # Deleting trailing slashes
            while [ "${FILE%/}" != "$FILE" ]
            do
                FILE="${FILE%/}"
            done
            shift 2
            if [ -e $FILE ]
            then
                if is_include_suitable $FILE
                then
                    EXCLUDE_LIST[$(( EXCLUDE_LIST_POS++ ))]=$FILE
                else
                    warning "Exclude option for file $FILE will be ignored (directories /dev, /proc, /sys are always excluded from archive)"
                fi
            else
                warning "Exclude option for file $FILE will be ignored (file does not exist)"
            fi
            ;;
        * )
            error "Unsupported option $1"
            ;;
    esac
done
unset -v PARSE_OPTIONS_SECTION

for (( i = 0; i < INCLUDE_LIST_POS; i++ ))
do
    info "File ${INCLUDE_LIST[$i]} will be included in guest file system archive"
done
for (( i = 0; i < EXCLUDE_LIST_POS; i++ ))
do
    info "File ${EXCLUDE_LIST[$i]} will be excluded from guest file system archive"
done


check_tool tar

# Searching name for archive
info "Searching for old archives ..."
declare ARCH_NAME=
if [ -e /guestfs.tgz ] || [ -e /guestfs_include.tgz ]
then
    if [ -e /guestfs.tgz ]
    then
        warning "Old archive /guestfs.tgz was found"
    fi
    if [ -e /guestfs_include.tgz ]
    then
        warning "Old archive /guestfs_include.tgz was found"
    fi
    for (( i = 0; ;i++ ))
    do
        if [ -e /guestfs_$i.tgz ]
        then
            warning "Old archive /guestfs_$i.tgz was found"
        fi
        if [ -e /guestfs_${i}_include.tgz ]
        then
            warning "Old archive /guestfs_${i}_include.tgz was found"
        fi
        if [ ! -e /guestfs_$i.tgz ] && [ ! -e /guestfs_${i}_include.tgz ]
        then
            typeset -r ARCH_NAME=guestfs_$i
            break
        fi
    done
    warning "Perhaps you should delete old archives or exclude them from packing"
else
    typeset -r ARCH_NAME=guestfs
fi

# Checking mount points
declare -a SENIOR_MOUNT_POINTS=()
declare -i SENIOR_MOUNT_POINTS_POS=0
declare MOUNT_DEVICES=":"
if [ -e /etc/mtab ]
then
    check_tool grep
    while read MOUNT_STR
    do
        set -- $MOUNT_STR
        MOUNT_POINT=$2
        if is_senior_mountpoint $MOUNT_POINT
        then
            MOUNT_DEVICE=$( stat -c %d $MOUNT_POINT )
            if ! grep -sq :$MOUNT_DEVICE: <<< "$MOUNT_DEVICES"
            then
                MOUNT_DEVICES="$MOUNT_DEVICES$MOUNT_DEVICE:"
                SENIOR_MOUNT_POINTS[$(( SENIOR_MOUNT_POINTS_POS++ ))]="$MOUNT_POINT"
            fi
        fi
    done < /etc/mtab
else
    warning "File /etc/mtab does not exist"
    warning "No information about mount points is available"
fi
info "List of mount points:"
for (( i = 0; i < SENIOR_MOUNT_POINTS_POS; i++ ))
do
    info "    ${SENIOR_MOUNT_POINTS[$i]}"
done

declare -r WORK_DIR=$PWD
cd /
if [ $INCLUDE_LIST_POS != 0 ]
then

    declare -a EXCLUDE2_LIST=()
    declare -i EXCLUDE2_LIST_POS=0

    for EXCL_FILE in ${EXCLUDE_LIST[@]}
    do
        for INCL_FILE in ${INCLUDE_LIST[@]}
        do
            if [ "${INCL_FILE}/" == "${EXCL_FILE:0:$(( ${#INCL_FILE} + 1 ))}" ]
            then
                EXCLUDE2_LIST[$(( EXCLUDE2_LIST_POS++ ))]=$EXCL_FILE
                break
            fi
        done
    done

    info "Creating include-files-archive /${ARCH_NAME}_include.tgz ..."
    info "tar -czf ${ARCH_NAME}_include.tgz ${EXCLUDE2_LIST[@]/#/--exclude } ${INCLUDE_LIST[@]}"
    if ! tar -czf ${ARCH_NAME}_include.tgz ${EXCLUDE2_LIST[@]/#/--exclude } ${INCLUDE_LIST[@]} >& /dev/null
    then
        rm -rf ${ARCH_NAME}_include.tgz
        error "Unable to create include-files-archive /${ARCH_NAME}_include.tgz"
    fi
fi

info "Creating guest file system archive /$ARCH_NAME.tgz ..."
info "tar -czf $ARCH_NAME.tgz --exclude $ARCH_NAME.tgz --exclude $SCRIPT ${EXCLUDE_LIST[@]/#/--exclude } ${SENIOR_MOUNT_POINTS[@]/#/--exclude } --exclude /. --exclude /.. /* /.*"
if ! tar -czf $ARCH_NAME.tgz --exclude $ARCH_NAME.tgz --exclude $SCRIPT ${EXCLUDE_LIST[@]/#/--exclude } ${SENIOR_MOUNT_POINTS[@]/#/--exclude } --exclude /. --exclude /.. /* /.* >& /dev/null
then
    rm -rf ${ARCH_NAME}_include.tgz
    rm -rf $ARCH_NAME.tgz
    error "Unable to create guest file system archive /$ARCH_NAME.tgz"
fi

if [ $INCLUDE_LIST_POS != 0 ]
then
    rm ${ARCH_NAME}_include.tgz
fi
warning "Guest file system archive /$ARCH_NAME.tgz was created"
cd $WORK_DIR
