#!/bin/bash

if declare -p __DEBUG__ >& /dev/null
then
    exec 2> /tmp/_$( basename $0 .sh )_$( id -u ).log
    set -x
fi

declare -r SCRIPT_DIR=$( dirname $( readlink -f $0 ) )
if ! source $SCRIPT_DIR/include 2> /dev/null
then
    if ! readlink --help >& /dev/null
    then
        echo "ERROR   : There is no readlink in $PATH"
        exit 2
    fi
    if ! dirname --help >& /dev/null
    then
        echo "ERROR   : There is no dirname in $PATH"
        exit 2
    fi
    echo "ERROR   : File $SCRIPT_DIR/include does not exist"
    exit 2
fi

declare -r SCRIPT=$( readlink -f $0 )

function print_help()
{
    if declare -p __SHOW_ACTION__ >& /dev/null
    then
        printf "\nunpack_guestfs.sh unpack guest file system archive created by pack_guestfs.sh\n\n"
    else
        printf "\n"
    fi
    printf "Usage:\n"
    printf "$SCRIPT [options] <archive-name>\n\n"
    printf "Options:\n"
    printf "\t-h, -H, --help                    print this help\n"
    printf "\t-v, --verbose                     turn on verbose mode\n"
    printf "\t--root <dir>                      set directory to unpack guest file system to (by default, current directory is used)\n"
    printf "\t--delete                          delete guest file system archive\n"
    printf "\n"

    return 0
}

declare GUEST_ROOT_DIR=$PWD

# Parsing options
declare PARSE_OPTIONS_SECTION=
test $# -gt 0 || error "No guest file system archive name is specified"
while [ ${1:0:1} == "-" ]
do
    case $1 in
        -h | -H | --help )
            __SHOW_ACTION__= print_help
            exit 0
            ;;
        -v | --verbose )
            unalias info >& /dev/null
            shift
            ;;
        --root )
            test $# -gt 1 || error "Directory for unpacking archive to is not specified"
            GUEST_ROOT_DIR=$2
            # Deleting trailing slashes
            while [ "${GUEST_ROOT_DIR%/}" != "$GUEST_ROOT_DIR" ]
            do
                GUEST_ROOT_DIR=${GUEST_ROOT_DIR%/}
            done
            shift 2
            ;;
        --delete )
            declare DELETE_MODE=
            shift
            ;;
        * )
            error "Unsupported option $1"
            ;;
    esac
    test $# -gt 0 || error "No guest file system archive name is specified"
done
declare -r ARCHIVE=$1
shift
test $# -eq 0 || error "No parameters after <archive-name> are possible ($@)"
unset -v PARSE_OPTIONS_SECTION

check_tool tar

# Checking archive
test -e $ARCHIVE || error "Archive $ARCHIVE does not exist"
test "${ARCHIVE%.tgz}" != "$ARCHIVE" || error "File $ARCHIVE is not a tgz-archive"

# Checking guest FS root directory
declare -r REAL_GUEST_ROOT_DIR=$( readlink -f $GUEST_ROOT_DIR )
test -e $REAL_GUEST_ROOT_DIR || error "Guest FS root $GUEST_ROOT_DIR does not exist"
test -d $REAL_GUEST_ROOT_DIR || error "Guest FS root $GUEST_ROOT_DIR is not a directory"
test -w $REAL_GUEST_ROOT_DIR || error "Guest FS root $GUEST_ROOT_DIR is not writable"
for FILE in $REAL_GUEST_ROOT_DIR/*
do
    if [ ! -e $FILE ]
    then
        break
    fi
    error "Guest FS root $GUEST_ROOT_DIR should not contain any files"
    break
done
GUEST_ROOT_DIR=$REAL_GUEST_ROOT_DIR

# Unpacking
declare ERROR=0
warning "Guest file system archive $ARCHIVE will be unpacked to directory $GUEST_ROOT_DIR"
info "Unpacking ..."
info "tar -C $GUEST_ROOT_DIR -xzf $ARCHIVE"
if ! tar -C $GUEST_ROOT_DIR -xzf $ARCHIVE
then
    warning "Guest file system may be incomplete due to problems with archive extraction"
    ERROR=1
fi
declare -r ARCHIVE_NAME=$( basename $ARCHIVE .tgz)
declare -r INCLUDE_ARCHIVE=${ARCHIVE_NAME}_include.tgz
if [ -e $GUEST_ROOT_DIR/$INCLUDE_ARCHIVE ]
then
    info "tar -C $GUEST_ROOT_DIR -xzf $GUEST_ROOT_DIR/$INCLUDE_ARCHIVE"
    if ! tar -C $GUEST_ROOT_DIR -xzf $GUEST_ROOT_DIR/$INCLUDE_ARCHIVE
    then
        warning "Guest file system may be incomplete due to problems with archive extraction"
        ERROR=1
    fi
    if declare -p DELETE_MODE >& /dev/null
    then
        info "Deleting guest file system include-archive $GUEST_ROOT_DIR/$INCLUDE_ARCHIVE"
        rm -rf $GUEST_ROOT_DIR/$INCLUDE_ARCHIVE
    fi
fi
if [ $ERROR == 0 ]
then
    warning "Guest file system was successfully unpacked"
    if declare -p DELETE_MODE >& /dev/null
    then
        info "Deleting guest file system archive $ARCHIVE"
        rm -rf $ARCHIVE
    fi
    mkdir -p $GUEST_ROOT_DIR/proc >& /dev/null
    mkdir -p $GUEST_ROOT_DIR/sys >& /dev/null
    mkdir -p $GUEST_ROOT_DIR/dev >& /dev/null
else
    warning "Guest file system was unpacked"
    warning "Option --delete will be ignored due to problems with unpacking"
fi
